@@ -21,7 +21,6 @@ from collections import deque
 from warnings import warn
 
 from gevent import sleep, spawn, get_hub
-from gevent.select import POLLIN, POLLOUT
 from ssh2.error_codes import LIBSSH2_ERROR_EAGAIN
 from ssh2.exceptions import SFTPHandleError, SFTPProtocolError, \
     Timeout as SSH2Timeout
@@ -35,7 +34,7 @@ from .tunnel import FORWARDER
 from ..base.single import BaseSSHClient
 from ...output import HostOutput
 from ...exceptions import SessionError, SFTPError, \
-    SFTPIOError, Timeout, SCPError, ProxyError, AuthenticationError
+    SFTPIOError, Timeout, SCPError, ProxyError
 from ...constants import DEFAULT_RETRIES, RETRY_DELAY
 
 
@@ -163,11 +162,14 @@ class SSHClient(BaseSSHClient):
         return proxy_local_port
 
     def disconnect(self):
-        """Disconnect session, close socket if needed."""
+        """Attempt to disconnect session.
+
+        Any errors on calling disconnect are suppressed by this function.
+        """
         self._keepalive_greenlet = None
         if self.session is not None:
             try:
-                self._eagain(self.session.disconnect)
+                self._disconnect_eagain()
             except Exception:
                 pass
             self.session = None
@@ -224,10 +226,7 @@ class SSHClient(BaseSSHClient):
             passphrase=password if password is not None else b'')
 
     def _password_auth(self):
-        try:
-            self.session.userauth_password(self.user, self.password)
-        except Exception as ex:
-            raise AuthenticationError("Password authentication failed - %s", ex)
+        self.session.userauth_password(self.user, self.password)
 
     def _open_session(self):
         chan = self._eagain(self.session.open_session)
@@ -316,10 +315,13 @@ class SSHClient(BaseSSHClient):
     def _eagain(self, func, *args, **kwargs):
         return self._eagain_errcode(func, LIBSSH2_ERROR_EAGAIN, *args, **kwargs)
 
+    def _make_sftp_eagain(self):
+        return self._eagain(self.session.sftp_init)
+
     def _make_sftp(self):
         """Make SFTP client from open transport"""
         try:
-            sftp = self._eagain(self.session.sftp_init)
+            sftp = self._make_sftp_eagain()
         except Exception as ex:
             raise SFTPError(ex)
         return sftp
@@ -486,6 +488,27 @@ class SSHClient(BaseSSHClient):
         logger.info("Copied local file %s from remote destination %s:%s",
                     local_file, self.host, remote_file)
 
+    def _scp_recv_recursive(self, remote_file, local_file, sftp, encoding='utf-8'):
+        try:
+            self._eagain(sftp.stat, remote_file)
+        except (SFTPHandleError, SFTPProtocolError):
+            msg = "Remote file or directory %s does not exist"
+            logger.error(msg, remote_file)
+            raise SCPError(msg, remote_file)
+        try:
+            dir_h = self._sftp_openfh(sftp.opendir, remote_file)
+        except SFTPError:
+            # remote_file is not a dir, scp file
+            return self.scp_recv(remote_file, local_file, encoding=encoding)
+        try:
+            os.makedirs(local_file)
+        except OSError:
+            pass
+        file_list = self._sftp_readdir(dir_h)
+        return self._scp_recv_dir(file_list, remote_file,
+                                  local_file, sftp,
+                                  encoding=encoding)
+
     def scp_recv(self, remote_file, local_file, recurse=False, sftp=None,
                  encoding='utf-8'):
         """Copy remote file to local host via SCP.
@@ -505,33 +528,13 @@ class SSHClient(BaseSSHClient):
           enabled.
         :type encoding: str
 
-        :raises: :py:class:`pssh.exceptions.SCPError` when a directory is
-          supplied to ``local_file`` and ``recurse`` is not set.
         :raises: :py:class:`pssh.exceptions.SCPError` on errors copying file.
         :raises: :py:class:`IOError` on local file IO errors.
         :raises: :py:class:`OSError` on local OS errors like permission denied.
         """
         if recurse:
             sftp = self._make_sftp() if sftp is None else sftp
-            try:
-                self._eagain(sftp.stat, remote_file)
-            except (SFTPHandleError, SFTPProtocolError):
-                msg = "Remote file or directory %s does not exist"
-                logger.error(msg, remote_file)
-                raise SCPError(msg, remote_file)
-            try:
-                dir_h = self._sftp_openfh(sftp.opendir, remote_file)
-            except SFTPError:
-                pass
-            else:
-                try:
-                    os.makedirs(local_file)
-                except OSError:
-                    pass
-                file_list = self._sftp_readdir(dir_h)
-                return self._scp_recv_dir(file_list, remote_file,
-                                          local_file, sftp,
-                                          encoding=encoding)
+            return self._scp_recv_recursive(remote_file, local_file, sftp, encoding=encoding)
         elif local_file.endswith('/'):
             remote_filename = remote_file.rsplit('/')[-1]
             local_file += remote_filename
@@ -561,11 +564,6 @@ class SSHClient(BaseSSHClient):
                     continue
                 total += size
                 local_fh.write(data)
-            if total != fileinfo.st_size:
-                msg = "Error copying data from remote file %s on host %s. " \
-                      "Copied %s out of %s total bytes"
-                raise SCPError(msg, remote_file, self.host, total,
-                               fileinfo.st_size)
         finally:
             local_fh.close()
             file_chan.close()
@@ -690,16 +688,12 @@ class SSHClient(BaseSSHClient):
         Blocks current greenlet only if socket has pending read or write operations
         in the appropriate direction.
         """
-        timeout = self.timeout if timeout is None else timeout
-        directions = self.session.block_directions()
-        if directions == 0:
-            return
-        events = 0
-        if directions & LIBSSH2_SESSION_BLOCK_INBOUND:
-            events = POLLIN
-        if directions & LIBSSH2_SESSION_BLOCK_OUTBOUND:
-            events |= POLLOUT
-        self._poll_socket(events, timeout=timeout)
+        self._poll_errcodes(
+            self.session.block_directions,
+            LIBSSH2_SESSION_BLOCK_INBOUND,
+            LIBSSH2_SESSION_BLOCK_OUTBOUND,
+            timeout=timeout,
+        )
 
     def _eagain_write(self, write_func, data, timeout=None):
         """Write data with given write_func for an ssh2-python session while
